package external;

import jade.content.AgentAction;
import jade.content.ContentManager;
import jade.content.lang.Codec;
import jade.content.lang.sl.SLCodec;
import jade.content.onto.Ontology;
import jade.content.onto.OntologyException;
import jade.content.onto.basic.Action;
import jade.core.AID;
import jade.core.Agent;
import jade.domain.FIPANames.InteractionProtocol;
import jade.lang.acl.ACLMessage;
import jade.proto.SimpleAchieveREInitiator;
import jade.util.leap.ArrayList;
import jade.util.leap.List;
import misc.AgentHelper;
import ontology.CICAgentDescription;
import ontology.CICDeregister;
import ontology.CICRegister;
import ontology.LogisticsOntology;
import ontology.Product;
import external.wha.OrderBehaviour;
import external.wha.ReminderBehaviour;

/**
 * 
 * Represents WhA in Logistics System. WhA models the role of the supplier.
 * It registers in CIC and responds to CFPs from OAs.
 * Finally it deregisters from the CIC when terminated.
 * 
 * Dependencies:
 * external.wha package
 * 
 */
public class WholesalerAgent extends Agent {
	private final int ACTION_REGISTER = 1;
	private final int ACTION_DEREGISTER = 2;

	public SLCodec contentLanguage;
	public Ontology contentOntology;
	private CICAgentDescription cicDescription;
	private ArrayList productList;
	private AID myCIC;

	protected void setup() {
		AgentHelper.debugMessage(this, "started");

		contentLanguage = new SLCodec();
		contentOntology = LogisticsOntology.getInstance();
		productList = setupProductList();
		cicDescription = setupDescription();

		ContentManager manager = getContentManager();
		manager.registerLanguage(contentLanguage);
		manager.registerOntology(contentOntology);

		// ** since CIC of the system must be known:
		myCIC = new AID("cic", false); // this is local name, if global, set
		// true
		// **

		ACLMessage request = createRequestMessage(myCIC, ACTION_REGISTER);

		addBehaviour(new RegisterBehaviour(this, request));
		addBehaviour(new OrderBehaviour(this));
		addBehaviour(new ReminderBehaviour(this));
	}

	protected void takeDown() {
		ACLMessage request = createRequestMessage(myCIC, ACTION_DEREGISTER);
		this.send(request);
	}

	protected ACLMessage createRequestMessage(AID recipient, int actionType) {
		ACLMessage request = new ACLMessage(ACLMessage.REQUEST);
		request.setSender(getAID());
		request.setLanguage(contentLanguage.getName());
		request.setOntology(contentOntology.getName());
		request.addReceiver(recipient);
		request.setProtocol(InteractionProtocol.FIPA_REQUEST);

		AgentAction content = null;

		switch (actionType) {
		case ACTION_REGISTER:
			CICRegister register = new CICRegister();
			register.setAgentCICDescription(cicDescription);
			content = register;
			break;
		case ACTION_DEREGISTER:
			CICDeregister deregister = new CICDeregister();
			deregister.setAgentAID(this.getAID());
			content = deregister;
			break;
		}

		Action action = new Action(recipient, content);

		try {
			getContentManager().fillContent(request, action);
			AgentHelper.debugMessage(this, "sending register request to "
					+ recipient.getLocalName());
		} catch (OntologyException e) {
			AgentHelper.debugMessage(this,
					"Error relating to message and ontology");
		} catch (Codec.CodecException e) {
			AgentHelper.debugMessage(this, "Error Parsing the message format");
		}
		return request;
	}

	private CICAgentDescription setupDescription() {
		CICAgentDescription description = new CICAgentDescription();
		description.setAgentAID(this.getAID());
		description.setOfferedProducts((List) productList);
		return description;
	}

	private ArrayList setupProductList() {
		ArrayList productList = new ArrayList();
		// TODO: create proper initializer reading from file or sth
		Product p = new Product();
		p.setProductName("someproduct");
		productList.add(p);
		return productList;
	}

	class RegisterBehaviour extends SimpleAchieveREInitiator {

		RegisterBehaviour(Agent agent, ACLMessage request) {
			super(agent, request);
		}

		protected void handleInform(ACLMessage msg) {
			AgentHelper.debugMessage(this.myAgent, msg.getSender()
					.getLocalName()
					+ " agreed to register me.");
		}

		public void reset(ACLMessage msg) {
			super.reset(msg);
		}

		protected void handleAgree(ACLMessage msg) {
			AgentHelper.debugMessage(this.myAgent, msg.getSender()
					.getLocalName()
					+ " agreed to register me.");
		}

		protected void handleRefuse(ACLMessage msg) {
			AgentHelper.debugMessage(this.myAgent, msg.getSender()
					.getLocalName()
					+ " refused to register me.");
		}

		protected void handleFailure(ACLMessage msg) {
			AgentHelper.debugMessage(this.myAgent, msg.getSender()
					.getLocalName()
					+ " failed to register me.");
		}

		protected void handleNotUnderstood(ACLMessage msg) {
			AgentHelper.debugMessage(this.myAgent, msg.getSender()
					.getLocalName()
					+ " did not understand my request");
		}

		protected void handleOutOfSequence(ACLMessage msg) {
			AgentHelper.debugMessage(this.myAgent, msg.getSender()
					.getLocalName()
					+ " sent me unexpected message");
		}
	}
}