package internal;

import internal.la.IssueOrderBehaviour;
import jade.content.ContentManager;
import jade.content.lang.sl.SLCodec;
import jade.content.onto.Ontology;
import jade.core.AID;
import jade.core.Agent;
import jade.util.leap.ArrayList;
import jade.util.leap.Iterator;

import java.util.Hashtable;

import misc.AgentHelper;
import ontology.LogisticsOntology;

/**
 * 
 * Represents LA in Logistics System. LA is responsible for managing ordering of
 * products and monitors their deliveries. Order requests and delivery
 * notifications are received from WA. Orders are dispatched to OAs.
 * 
 * Dependencies:
 * AgentHelper.extractOrderDescription
 * AgentHelper.extractOrderConfirmation AgentHelper.extractDelivery
 * internal.la package
 * 
 */
public class LogisticsAgent extends Agent {

	// constants for reliability management
	private final int RELIABILITY_INIT = 100;
	private final int RELIABILITY_BONUS = 10;
	private final int RELIABILITY_PENALTY = 10;

	// real time length of each 'tick' (delivery times are expressed in 'ticks')
	public final int MONITORING_INTERVAL = 500;

	public SLCodec contentLanguage;
	public Ontology contentOntology;

	public Hashtable reliabilityList; // ranks of known suppliers
	public ArrayList oaPool; // pool of OAs serving our requests
	public AID myCIC; // CIC of our system
	public AID myWA; // our WA

	// initialization
	protected void setup() {
		AgentHelper.debugMessage(this, "created");

		contentLanguage = new SLCodec();
		contentOntology = LogisticsOntology.getInstance();
		ContentManager manager = getContentManager();
		manager.registerLanguage(contentLanguage);
		manager.registerOntology(contentOntology);

		reliabilityList = new Hashtable();

		// ** initialize OA pool - all are known at creation time
		oaPool = new ArrayList();
		oaPool.add(new AID("oa1", false)); // if global name set true
		oaPool.add(new AID("oa2", false));
		// **

		// ** initialize CIC - must be known at creation time
		myCIC = new AID("cic", false); // if global name set true
		// **

		// ** initialize WA - must be known at creation time
		myWA = new AID("wa", false); // if global name set true
		// **

		// allow LA to serve requests
		addBehaviour(new IssueOrderBehaviour(this));
	}

	// get reliability for supplier
	public int getReliability(AID wha) {
		// try get value from our list
		Integer rel = (Integer) reliabilityList.get(wha);
		int reliability;
		if (rel != null)
			// value found, set result
			reliability = rel.intValue();
		else {
			// value not found = new supplier
			// set result to initial reliability
			reliability = RELIABILITY_INIT;
			// remember this supplier
			reliabilityList.put(wha, new Integer(reliability));
		}
		return reliability;
	}

	// manage reliability (see docs for human-friendly procedure description)
	public void manageReliability(ArrayList suppliers, boolean success,
			boolean beforeDeadline) {
		AID wha = null;
		Integer oldRel = null;

		AgentHelper.debugMessage(this, "managing reliability");

		if (suppliers.size() == 0)
			// there were no suppliers involved, nothing to do
			return;

		// last involved supplier - special treatment
		wha = (AID) suppliers.remove(suppliers.size() - 1);

		// first manage last supplier on the list
		if (success) {
			// order was successful
			// get previous reliability
			oldRel = (Integer) reliabilityList.get(wha);
			// award bonus and save
			reliabilityList.put(wha, new Integer(oldRel.intValue()
					+ RELIABILITY_BONUS));
			AgentHelper.debugMessage(this, "bonus: " + wha.getLocalName());
		} else if (!beforeDeadline) {
			// order failed
			// get previous reliability
			oldRel = (Integer) reliabilityList.get(wha);
			// give penalty and save
			reliabilityList.put(wha, new Integer(oldRel.intValue()
					- RELIABILITY_PENALTY));
			AgentHelper.debugMessage(this, "penalty: " + wha.getLocalName());
		}

		// all ther supliers failed us, so penalties across the board
		for (Iterator iter = suppliers.iterator(); iter.hasNext();) {
			wha = (AID) iter.next();
			oldRel = (Integer) reliabilityList.get(wha);
			reliabilityList.put(wha, new Integer(oldRel.intValue()
					- RELIABILITY_PENALTY));
			AgentHelper.debugMessage(this, "penalty: " + wha.getLocalName());
		}
	}
}