package internal.la;

import ontology.DeliveryDescription;
import ontology.OrderConfirmation;
import internal.LogisticsAgent;
import jade.core.behaviours.Behaviour;
import jade.core.behaviours.ParallelBehaviour;
import jade.core.behaviours.SimpleBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.proto.states.MsgReceiver;
import misc.AgentHelper;

// handles logic of waiting for the delivery to come
public class MonitorBehaviour extends ParallelBehaviour {
	IssueOrderBehaviour theMaster;
	Behaviour deliveryListener = null;
	private int result = 1;
	private final LogisticsAgent theAgent;

	// behaviour to keep the Monitor Behaviour running, without placing
	// addtional code for that in delivery listener
	private class Dummy extends SimpleBehaviour {
		public void action() {
			block(1000);
		}

		// never complete, but is blocked all the time, so no busy waiting
		public boolean done() {
			return false;
		}
	}

	public MonitorBehaviour(IssueOrderBehaviour master) {
		super(master.theAgent, ParallelBehaviour.WHEN_ANY);
		theMaster = master;
		theAgent = theMaster.theAgent;
		// Dummy is always running
		addSubBehaviour(new Dummy());
	}

	// when behaviour is about to start
	public void onStart() {
		// reset available OA list after ordering
		theMaster.currentOA = 0;
		// prepare template for WA delivery messages
		MessageTemplate mt = MessageTemplate.and(MessageTemplate
				.MatchSender(theAgent.myWA), MessageTemplate.and(MessageTemplate
				.MatchPerformative(ACLMessage.INFORM), MessageTemplate.and(
				MessageTemplate.MatchOntology(theAgent.contentOntology.getName()),
				MessageTemplate.MatchLanguage(theAgent.contentLanguage.getName()))));
		// if not null, this not the first time this behaviour starts
		// so remove it, to dispose its state
		if (deliveryListener != null)
			removeSubBehaviour(deliveryListener);
		// create new dalivery listener behaviour (generic msg receiver)
		deliveryListener = new MsgReceiver(theMaster.theAgent, mt, System
				.currentTimeMillis()
				+ theMaster.theTimeout * theAgent.MONITORING_INTERVAL,
				theMaster.data, "delivery");
		// add it to monitoring behaviour
		this.addSubBehaviour(deliveryListener);
	}

	// see if monitoring is done
	protected boolean checkTermination(boolean currentDone,
			int currentResult) {
		// if subbehaviour calling this is not done itself, we are obviously
		// not done
		// e.g. Dummy
		if (currentDone == false)
			return false;
		// otherwise its MsgReceiver calling
		if (currentResult == MsgReceiver.TIMEOUT_EXPIRED) {
			// the reason is timeout
			AgentHelper.debugMessage(theAgent,
					"delivery message timeout expired");
			// deadline is coming closer
			theMaster.theDeadline -= theMaster.theTimeout;
			// check deadline
			if (theMaster.theDeadline <= 0) {
				// it's crossed, that's it then
				result = 2;
				AgentHelper.debugMessage(theAgent, "failure");
			} else {
				// otherwise we have a chance to remind or make new order
				if (theMaster.reminderMessage == null) {
					// we didn't remind yet, so let's do now
					result = 4;
					AgentHelper.debugMessage(theAgent,
							"try again: remind");
				} else {
					// we did remind, so let's find a better supplier
					result = 1;
					AgentHelper.debugMessage(theAgent,
							"try again: new order");
					// and reset the reminder, we start out fresh
					theMaster.reminderMessage = null;
				}
			}
			// anyway we need to remind, reorder or quit now, so monitoring
			// is complete
			return true;
		} else {
			// the reason is the delivery message
			AgentHelper.debugMessage(theAgent,
					"delivery message received");
			// extract the delivery
			theMaster.theDelivery = LAHelper.extractDelivery(theAgent, theMaster
					.retrieveData("delivery"));
			// check if delivery fits the confirmation
			if (verifyDelivery(theMaster.theConfirmation,
					theMaster.theDelivery)) {
				// it does
				AgentHelper.debugMessage(theAgent, "delivery correct");
				result = 0;
				return true;
			} else {
				// its some other delivery, not for us
				AgentHelper
						.debugMessage(theAgent, "delivery incorrect");
				this.reset();
				return false;
			}
		}
	}

	// result for FSM
	public int onEnd() {
		return result;
	}
	
	// verify delivery details against the confirmation
	private boolean verifyDelivery(OrderConfirmation conf,
			DeliveryDescription desc) {
		if (desc == null || conf == null)
			// we are missing some data, so can't verify
			return false;
		if (conf.getOrderId().compareTo(desc.getOrderId()) == 0) {
			// order id matches
			return true;
		} else {
			// this is some other delivery
			return false;
		}
	}
}
