package internal.la;

import internal.LogisticsAgent;
import jade.content.AgentAction;
import jade.content.lang.Codec.CodecException;
import jade.content.onto.OntologyException;
import jade.content.onto.basic.Action;
import jade.core.AID;
import jade.lang.acl.ACLMessage;
import jade.proto.SimpleAchieveREInitiator;

import java.util.Vector;

import ontology.InformResult;
import ontology.OrderRequestResult;
import ontology.Result;
import ontology.ResultFailure;
import ontology.ResultSuccess;

import misc.AgentHelper;

// handles ordering with OA
public class OrderBehaviour extends SimpleAchieveREInitiator {
	private int success = 1;

	private IssueOrderBehaviour theMaster;
	private final LogisticsAgent theAgent;

	private ACLMessage theMessage; // the message we sent and was accepted
	private AID oa; // OA we chose as target of message

	// sends the optional ordering result notification message to WA
	private void sendMessage(int success) {
		ACLMessage orderingResult = prepareOrderingResult(success,
				theMaster);
		myAgent.send(orderingResult);
	}

	public OrderBehaviour(IssueOrderBehaviour master, ACLMessage request) {
		super(master.theAgent, request);
		theMaster = master;
		theAgent = theMaster.theAgent;
	}

	// prepares issue order message
	protected ACLMessage prepareRequest(ACLMessage msg) {
		// reset confirmation, before new one is received (so we know when
		// it isn't)
		theMaster.theConfirmation = null;
		msg = theMaster.issueOrderTemplate;
		if (msg == null) {
			// somehow we don't have a template ready
			AgentHelper.errorMessage(this.myAgent,
					"issue order template not ready!");
			return null;
		}
		if (theMaster.currentOA < theMaster.theAgent.oaPool.size()) {
			// if we still have OAs available
			msg.clearAllReceiver();
			// address the message to the current OA
			oa = (AID) theMaster.theAgent.oaPool.get(theMaster.currentOA++);
			msg.addReceiver(oa);
			// save the actual ACLMessage we are going to send
			theMessage = msg;
			AgentHelper.debugMessage(this.myAgent,
					"Requesting ordering from " + oa.getLocalName());
			return msg;
		} else {
			// no more OAs available
			// wait for them and retry from the first
			theMaster.currentOA = 0;
			block(theAgent.MONITORING_INTERVAL);
			msg.clearAllReceiver();
			// address the message to the current OA
			oa = (AID) theMaster.theAgent.oaPool.get(theMaster.currentOA++);
			msg.addReceiver(oa);
			// save the actual ACLMessage we are going to send
			theMessage = msg;
			AgentHelper.debugMessage(this.myAgent,
					"Requesting ordering from " + oa.getLocalName());
			return msg;
		}
	}

	public void reset(ACLMessage msg) {
		super.reset(msg);
	}

	protected void handleAgree(ACLMessage msg) {
		AgentHelper.debugMessage(this.myAgent, msg.getSender()
				.getLocalName()
				+ " accepted the order");
	}

	// OA refused - perhaps it is busy
	protected void handleRefuse(ACLMessage msg) {
		AgentHelper.debugMessage(this.myAgent, msg.getSender()
				.getLocalName()
				+ " is busy");
		success = 1;
	}

	// handle timeout
	protected void handleAllResponses(Vector msgs) {
		AgentHelper.debugMessage(this.myAgent, oa.getLocalName()
				+ " did not respond, maybe it's busy");
		success = 1;
	}

	// OA responded successfully
	protected void handleInform(ACLMessage msg) {
		AgentHelper.debugMessage(this.myAgent, msg.getSender()
				.getLocalName()
				+ " ordered successfully");
		// get the confirmation
		theMaster.theConfirmation = LAHelper.extractOrderConfirmation(myAgent, msg);
		// save the message we sent
		theMaster.issueOrderMessage = theMessage;
		// set delivery timeout as promised in confirmation
		theMaster.theTimeout = theMaster.theConfirmation
				.getOfferDescription().getDeliveryTimeSpecific();
		success = 0;
		// save chosen supplier in our list of involved
		AID chosenSupplier = theMaster.theConfirmation
				.getOfferDescription().getAgentAID();
		theMaster.involvedSuppliers.add(chosenSupplier);
		// remove supplier from potential suppliers, so we won't call him
		// again if he fails
		theMaster.removeSupplier(theMaster, chosenSupplier);
		// scenario switch - send notification message, so WA replies with
		// delivery notification
		if (AgentHelper.scenario == 2)
			sendMessage(1);
	}

	protected void handleNotUnderstood(ACLMessage msg) {
		AgentHelper.debugMessage(this.myAgent, msg.getSender()
				.getLocalName()
				+ " did not understand my request");
	}

	protected void handleOutOfSequence(ACLMessage msg) {
		AgentHelper.debugMessage(this.myAgent, msg.getSender()
				.getLocalName()
				+ " sent me unexpected message");
	}

	// OA failed to order, i.e. no suppliers willing to sell - fail the
	// order
	protected void handleFailure(ACLMessage msg) {
		AgentHelper.debugMessage(this.myAgent, msg.getSender()
				.getLocalName()
				+ " failed to order");
		success = 2;
		// sendMessage(success);
	}

	// FSM result
	public int onEnd() {
		return success;
	}
	
	// prepare optional ordering result notification for WA
	private ACLMessage prepareOrderingResult(int success,
			IssueOrderBehaviour theMaster) {
		LogisticsAgent theAgent = theMaster.theAgent;
		// reply to the original WA message
		ACLMessage message = theMaster.orderRequestMessage.createReply();

		Result result = new Result();
		if (success == 0) {
			// ordering success - not used (would terminate protocol)
			result.setResultType(new ResultSuccess());
			message.setPerformative(ACLMessage.INFORM);
		} else if (success == 1) {
			// optional ordering success notification
			OrderRequestResult orr = new OrderRequestResult();
			orr.setOrderConfirmation(theMaster.theConfirmation);
			result = orr;
			message.setPerformative(ACLMessage.AGREE);
		} else {
			// order failure
			result.setResultType(new ResultFailure());
			message.setPerformative(ACLMessage.FAILURE);
		}
		InformResult inform = new InformResult();
		inform.setResult(result);

		AgentAction content = inform;
		Action action = new Action(theAgent.myWA, content);

		try {
			// fill contents of inform
			theAgent.getContentManager().fillContent(message, action);
		} catch (CodecException e) {
			AgentHelper.errorMessage(theAgent, "Error relating to message and"
					+ "ontology");
			return null;
		} catch (OntologyException e) {
			AgentHelper.errorMessage(theAgent, "Error parsing the message format");
			return null;
		}

		return message;
	}
}
