package internal.oa;

import internal.OrderingAgent;
import jade.content.AgentAction;
import jade.content.lang.Codec.CodecException;
import jade.content.onto.OntologyException;
import jade.content.onto.basic.Action;
import jade.core.AID;
import jade.core.behaviours.SimpleBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import misc.AgentHelper;
import ontology.ConfirmationRequest;
import ontology.OfferDescription;
import ontology.OrderConfirmation;

// handles sending confirmation requests and receiving confirmations from WhAs
// who answered our CFP with proposals.
public class ConfirmationBehaviour extends SimpleBehaviour {

	private final OrderingBehaviour theMaster;
	private int result = 1;
	private final OrderingAgent theAgent;

	public ConfirmationBehaviour(OrderingBehaviour master) {
		this.theMaster = master;
		theAgent = theMaster.theAgent;
	}

	public void action() {
		AgentHelper.debugMessage(this.myAgent, "confirming offer");

		// get offer with the best SRF
		int bestOffer = getBestOfferIndex(theMaster);
		if (bestOffer == -1) {
			// no offers left - quit
			result = 2;
			return;
		}
		ACLMessage offerMsg = (ACLMessage) theMaster.proposeMessages
				.get(bestOffer);
		if (offerMsg == null) {
			// offer not present (error) - quit
			result = 2;
			return;
		}

		// winner agent
		AID winner = offerMsg.getSender();

		// set up a reply to the original propose
		ACLMessage accept = offerMsg.createReply();
		accept.setPerformative(ACLMessage.ACCEPT_PROPOSAL);

		ConfirmationRequest confReq = new ConfirmationRequest();
		OfferDescription offer = (OfferDescription) theMaster.offerList
				.get(bestOffer);
		confReq.setOfferDescription(offer);

		AgentAction respContent = confReq;
		Action action = new Action(winner, respContent);

		try {
			// fill contents of response
			theAgent.getContentManager().fillContent(accept, action);
		} catch (CodecException e) {
			System.err.println("Error relating to message and " + "ontology: "
					+ e);
		} catch (OntologyException e) {
			System.err.println(myAgent.getLocalName()
					+ ":Error Parsing the message format: " + e);
		}
		// send the message
		myAgent.send(accept);
		// prepare template for the expected response
		MessageTemplate mt = MessageTemplate.and(MessageTemplate
				.MatchSender(winner), MessageTemplate.and(MessageTemplate
				.MatchOntology(theAgent.contentOntology.getName()),
				MessageTemplate.MatchLanguage(theAgent.contentLanguage
						.getName())));
		// wait for the response
		ACLMessage response = theAgent.blockingReceive(mt,
				OrderingAgent.RESPONSE_TIMEOUT);
		if (response != null && response.getPerformative() == ACLMessage.INFORM) {
			// response came and it's not refusal/failure
			OrderConfirmation conf = OAHelper.extractConfirmation(theAgent,
					response);
			if (OAHelper.verifyConfirmation(conf, offer) == true) {
				// confirmation is ok
				theMaster.theConfirmation = conf;
				result = 0;
			} else {
				// confirmation is not ok
				// remove the faulty offer
				theMaster.removeOffer(bestOffer, theMaster);
			}
		} else {
			// supplier didnt answer in time - remove the faulty offer
			theMaster.removeOffer(bestOffer, theMaster);
		}
	}

	// are we done?
	public boolean done() {
		if (result == 1)
			// no confirmation yet, but still proposals left
			return false;
		else
			// ordered or no more offers to consider
			return true;
	}

	// set result for FSM
	public int onEnd() {
		if (result == 0)
			// confirmation succesful
			return 0;
		else
			// confirmation failed
			return 1;
	}

	// retrieves index of best offer in the lists
	private int getBestOfferIndex(OrderingBehaviour theMaster) {
		int bestIndex = -1;
		Double bestRank = null;
		// simple search
		for (int i = 0; i < theMaster.offerRanks.size(); i++) {
			Double current = (Double) theMaster.offerRanks.get(i);
			if (i == 0) {
				bestRank = current;
				bestIndex = i;
			} else {
				if (current.compareTo(bestRank) > 0) {
					bestRank = current;
					bestIndex = i;
				}
			}
		}
		// and debug
		AgentHelper.debugMessage(theAgent, "best rank is "
				+ bestRank.toString()
				+ " from "
				+ ((ACLMessage) theMaster.proposeMessages.get(bestIndex))
						.getSender().getLocalName());
		return bestIndex;
	}
}
