package internal.oa;

import java.util.Iterator;

import jade.content.Concept;
import jade.content.ContentElement;
import jade.content.lang.Codec;
import jade.content.onto.OntologyException;
import jade.content.onto.basic.Action;
import jade.core.Agent;
import jade.domain.FIPAAgentManagement.NotUnderstoodException;
import jade.lang.acl.ACLMessage;
import jade.util.leap.ArrayList;
import misc.AgentHelper;
import ontology.CFPResponse;
import ontology.ConfirmationResponse;
import ontology.IssueOrder;
import ontology.LAAgentDescription;
import ontology.OfferDescription;
import ontology.OrderConfirmation;
import ontology.OrderDescription;
import ontology.Reminder;

/**
 * 
 * Contains helper methods for OA and its behaviours.
 *
 */
public class OAHelper {
	
	// verifies that WhA confirmed what we wanted
	public static boolean verifyConfirmation(OrderConfirmation confirm,
			OfferDescription offer) {
		OfferDescription proposal = confirm.getOfferDescription();
		if (proposal.getAgentAID().compareTo(offer.getAgentAID()) != 0)
			return false;
		if (proposal.getAmountSpecific() != offer.getAmountSpecific())
			return false;
		if (proposal.getDeliveryTimeSpecific() != offer.getDeliveryTimeSpecific())
			return false;
		if (proposal.getPriceSpecific() != offer.getPriceSpecific())
			return false;
		if (proposal.getTheProduct().getProductName().compareTo(
				offer.getTheProduct().getProductName()) != 0)
			return false;
		return true;
	}
	
	// extracts confirmation from order confirmations and reminder responses
	public static OrderConfirmation extractConfirmation(Agent agent, ACLMessage msg) {
		ContentElement content = null;

		try {
			try {
				content = agent.getContentManager().extractContent(msg);
			} catch (OntologyException e) {
				AgentHelper.errorMessage(agent, "Error relating to message and "
						+ "ontology");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			} catch (Codec.CodecException e) {
				AgentHelper.errorMessage(agent,
						"Error Parsing the message format");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			}

			Concept action_content;

			if (content == null || !(content instanceof Action)) {
				AgentHelper.errorMessage(agent, "Message body was not an action");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			}

			action_content = ((Action) content).getAction();

			if (action_content == null) {
				AgentHelper.errorMessage(agent, "Message body was not present");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			} else {
				if (action_content instanceof ConfirmationResponse) {
					// we have order confirmation
					ConfirmationResponse information_request = (ConfirmationResponse) action_content;

					return information_request.getOrderConfirmation();
				} else if (action_content instanceof Reminder) {
					// we have reminder response
					Reminder information_request = (Reminder) action_content;

					return information_request.getOrderConfirmation();
				} else {
					// or something unidentified
					AgentHelper.errorMessage(agent,
							"Message body was not a correct action");
					throw new NotUnderstoodException("(" + msg.toString() + ")");
				}
			}
		} catch (NotUnderstoodException e) {
			;
		}
		return null;
	}
	
	// extract order from LA command
	public static OrderDescription extractOrder(Agent theAgent, ACLMessage msg) {
		ContentElement content = null;

		try {
			try {
				content = theAgent.getContentManager().extractContent(msg);
			} catch (OntologyException e) {
				AgentHelper.errorMessage(theAgent, "Error relating to message and "
						+ "ontology");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			} catch (Codec.CodecException e) {
				AgentHelper.errorMessage(theAgent,
						"Error Parsing the message format");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			}

			Concept action_content;

			if (content == null || !(content instanceof Action)) {
				AgentHelper.errorMessage(theAgent, "Message body was not an action");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			}

			action_content = ((Action) content).getAction();

			if (action_content == null) {
				AgentHelper.errorMessage(theAgent, "Message body was not present");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			} else {
				if (action_content instanceof IssueOrder) {

					IssueOrder information_request = (IssueOrder) action_content;

					// get order details and return them
					OrderDescription theDescription = information_request.getOrderDescription();
					return theDescription;
				} else {
					AgentHelper.errorMessage(theAgent,
							"Message body was not a correct action");
					throw new NotUnderstoodException("(" + msg.toString() + ")");
				}
			}
		} catch (NotUnderstoodException e) {
			;
		}
		return null;
	}
	
	// extract offer from propose message
	public static OfferDescription extractOffer(Agent theAgent, ACLMessage msg) {
		ContentElement content = null;

		try {
			try {
				content = theAgent.getContentManager().extractContent(msg);
			} catch (OntologyException e) {
				AgentHelper.errorMessage(theAgent, "Error relating to message and "
						+ "ontology");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			} catch (Codec.CodecException e) {
				AgentHelper.errorMessage(theAgent,
						"Error Parsing the message format");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			}

			Concept action_content;

			if (content == null || !(content instanceof Action)) {
				AgentHelper.errorMessage(theAgent, "Message body was not an action");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			}

			action_content = ((Action) content).getAction();

			if (action_content == null) {
				AgentHelper.errorMessage(theAgent, "Message body was not present");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			} else {
				if (action_content instanceof CFPResponse) {
					CFPResponse information_request = (CFPResponse) action_content;

					return information_request.getOfferDescription();
				} else {
					AgentHelper.errorMessage(theAgent,
							"Message body was not a correct action");
					throw new NotUnderstoodException("(" + msg.toString() + ")");
				}
			}
		} catch (NotUnderstoodException e) {
			;
		}
		return null;
	}
	
	// extract suppliers from LA command
	public static ArrayList extractSuppliers(Agent theAgent, ACLMessage msg) {
		ContentElement content = null;
		ArrayList theSuppliers = null;

		try {
			try {
				content = theAgent.getContentManager().extractContent(msg);
			} catch (OntologyException e) {
				AgentHelper.errorMessage(theAgent, "Error relating to message and "
						+ "ontology");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			} catch (Codec.CodecException e) {
				AgentHelper.errorMessage(theAgent,
						"Error Parsing the message format");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			}

			Concept action_content;

			if (content == null || !(content instanceof Action)) {
				AgentHelper.errorMessage(theAgent, "Message body was not an action");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			}

			action_content = ((Action) content).getAction();

			if (action_content == null) {
				AgentHelper.errorMessage(theAgent, "Message body was not present");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			} else {
				if (action_content instanceof IssueOrder) {

					IssueOrder information_request = (IssueOrder) action_content;

					// get suppliers from LA
					LAAgentDescription desc = null;
					theSuppliers = new ArrayList();
					for (Iterator iter = information_request.getAllAgentLADescriptions(); iter.hasNext();) {
						desc = (LAAgentDescription) iter.next();
						theSuppliers.add(desc);
					}
				} else {
					AgentHelper.errorMessage(theAgent,
							"Message body was not a correct action");
					throw new NotUnderstoodException("(" + msg.toString() + ")");
				}
			}
		} catch (NotUnderstoodException e) {
			;
		}
		return theSuppliers;
	}
}