package internal.oa;

import internal.OrderingAgent;
import internal.oa.ConfirmationBehaviour;
import internal.oa.DispatchBehaviour;
import internal.oa.FinalStateBehaviour;
import internal.oa.ProposalBehaviour;
import internal.oa.RankingBehaviour;
import internal.oa.ReminderBehaviour;
import internal.oa.SendCFPBehaviour;
import jade.core.Agent;
import jade.core.behaviours.DataStore;
import jade.core.behaviours.FSMBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.proto.states.MsgReceiver;
import jade.util.leap.ArrayList;

import java.util.Vector;

import misc.AgentHelper;
import ontology.DeliveryDescription;
import ontology.OfferDescription;
import ontology.OrderConfirmation;
import ontology.OrderDescription;

/** 
 * 
 * Main behaviour of OA.
 *
 */ 
public class OrderingBehaviour extends FSMBehaviour {
	// state name constants
	private final static String STATE_LACOMMAND = "laCommand";
	private final static String STATE_DISPATCH = "dispatch";
	private final static String STATE_CFP = "cfp";
	private final static String STATE_PROPOSALS = "proposals";
	private final static String STATE_RANKING = "rankOffers";
	private final static String STATE_CONFIRMATION = "confirmation";
	private final static String STATE_REMINDER = "reminder";
	private final static String STATE_FINAL = "final";
	
	public OrderingAgent theAgent = (OrderingAgent) myAgent;
	public ArrayList theSuppliers; // list of suppliers received from LA (<LAAgentDescription>)
	public OrderDescription theOrder; // order received from LA
	public OrderConfirmation theConfirmation; // confirmation from WhA
	public ACLMessage laCommandMessage; // message from LA
	public ACLMessage cfpTemplate;
	public Vector proposeMessages; // proposals received in reply to cfp
	public ArrayList offerList; // list of offers
	public ArrayList offerRanks; // ranks for those offers
	public DataStore data = new DataStore(); // data store for generic message receivers

	public DeliveryDescription theDelivery = null;

	// retrieve message identified by the key from data store
	public ACLMessage retrieveData(String key) {
		return (ACLMessage) data.get(key);
	}

	// initialization
	public OrderingBehaviour(Agent agent) {
		super(agent);

		// empty at first
		offerList = new ArrayList();
		offerRanks = new ArrayList();
		proposeMessages = new Vector();

		AgentHelper.debugMessage(this.myAgent, "configuring FSM");

		// template for messages from LA
		MessageTemplate mt = MessageTemplate.and(
				MessageTemplate.MatchSender(theAgent.myLA),
				MessageTemplate.and(
						MessageTemplate.MatchOntology(theAgent.contentOntology.getName()),
						MessageTemplate.MatchLanguage(theAgent.contentLanguage.getName())));

		// *** set FSM states
		registerFirstState(new MsgReceiver(agent, mt, MsgReceiver.INFINITE,
				data, "lacommand"), STATE_LACOMMAND); // receive LA command
		registerState(new DispatchBehaviour(this), STATE_DISPATCH); // decide what to do with it
		registerState(new SendCFPBehaviour(this), STATE_CFP); // send cfps
		registerState(new ProposalBehaviour(this), STATE_PROPOSALS); // receive proposals
		registerState(new RankingBehaviour(this), STATE_RANKING); // renk them
		registerState(new ConfirmationBehaviour(this), STATE_CONFIRMATION); // confirm order
		registerState(new ReminderBehaviour(this, laCommandMessage), // remind
				STATE_REMINDER);
		registerLastState(new FinalStateBehaviour(this), STATE_FINAL); // finish

		// *** set FSM transitions
		registerDefaultTransition(STATE_LACOMMAND, STATE_DISPATCH); // command received
		registerDefaultTransition(STATE_DISPATCH, STATE_FINAL); // uknown command
		registerDefaultTransition(STATE_CFP, STATE_FINAL); // cfp error
		registerDefaultTransition(STATE_PROPOSALS, STATE_FINAL); // no proposals
		registerDefaultTransition(STATE_RANKING, STATE_CONFIRMATION); // offers ranked
		registerDefaultTransition(STATE_CONFIRMATION, STATE_FINAL); // no confirmations
		registerDefaultTransition(STATE_REMINDER, STATE_FINAL); // reminder sent and response received (or not)

		registerTransition(STATE_DISPATCH, STATE_CFP, 0); // order command
		registerTransition(STATE_DISPATCH, STATE_REMINDER, 1); // reminder command

		registerTransition(STATE_CFP, STATE_PROPOSALS, 0); // sent out cfps
		registerTransition(STATE_PROPOSALS, STATE_RANKING, 0); // some proposals received

		registerTransition(STATE_CONFIRMATION, STATE_FINAL, 0); // confirmation received
		registerTransition(STATE_CONFIRMATION, STATE_CONFIRMATION, 1,
				new String[] { STATE_CONFIRMATION}); // no confirmation, try again
	}
	
	// remove all offer-related entries in one call
	public void removeOffer(int index, OrderingBehaviour theMaster) {
		theMaster.offerList.remove(index);
		theMaster.offerRanks.remove(index);
		theMaster.proposeMessages.remove(index);
	}

	// add all offer-related entries in one call
	public void addOffer(OfferDescription offer, ACLMessage message,
			OrderingBehaviour theMaster) {
		theMaster.offerList.add(offer);
		theMaster.proposeMessages.add(message);
		// offer rank is calculated and added in RankingBehaviour
	}
}
